@@ -12,7 +12,7 @@
 #pragma GCC poison PETSC_PETSC_HAVE_CXX_DIALECT_CXX11
 #pragma GCC poison PETSC_PETSC_HAVE_CXX_DIALECT_CXX14
 #pragma GCC poison PETSC_PETSC_HAVE_CXX_DIALECT_CXX17
-#pragma GCC poison PETSC_PETSC_HAVE_CXX_DIALECT_CXX20
+#pragma GCC poison PETSC_PETSC_HAVE_DEVICE
 #pragma GCC poison PETSC_PETSC_HAVE_DLADDR
 #pragma GCC poison PETSC_PETSC_HAVE_DLCLOSE
 #pragma GCC poison PETSC_PETSC_HAVE_DLERROR
@@ -42,12 +42,15 @@
 #pragma GCC poison PETSC_PETSC_HAVE_GETRUSAGE
 #pragma GCC poison PETSC_PETSC_HAVE_GSL
 #pragma GCC poison PETSC_PETSC_HAVE_HDF5
+#pragma GCC poison PETSC_PETSC_HAVE_HWLOC
 #pragma GCC poison PETSC_PETSC_HAVE_HYPRE
 #pragma GCC poison PETSC_PETSC_HAVE_IMMINTRIN_H
 #pragma GCC poison PETSC_PETSC_HAVE_INTTYPES_H
 #pragma GCC poison PETSC_PETSC_HAVE_ISINF
 #pragma GCC poison PETSC_PETSC_HAVE_ISNAN
 #pragma GCC poison PETSC_PETSC_HAVE_ISNORMAL
+#pragma GCC poison PETSC_PETSC_HAVE_KOKKOS
+#pragma GCC poison PETSC_PETSC_HAVE_KOKKOS_WITHOUT_GPU
 #pragma GCC poison PETSC_PETSC_HAVE_LGAMMA
 #pragma GCC poison PETSC_PETSC_HAVE_LIBJPEG
 #pragma GCC poison PETSC_PETSC_HAVE_LINUX
@@ -89,6 +92,7 @@
 #pragma GCC poison PETSC_PETSC_HAVE_OPENMPI
 #pragma GCC poison PETSC_PETSC_HAVE_PACKAGES
 #pragma GCC poison PETSC_PETSC_HAVE_PARMETIS
+#pragma GCC poison PETSC_PETSC_HAVE_PASTIX
 #pragma GCC poison PETSC_PETSC_HAVE_PETSC4PY
 #pragma GCC poison PETSC_PETSC_HAVE_POPEN
 #pragma GCC poison PETSC_PETSC_HAVE_POSIX_MEMALIGN
@@ -107,7 +111,6 @@
 #pragma GCC poison PETSC_PETSC_HAVE_RTLD_LOCAL
 #pragma GCC poison PETSC_PETSC_HAVE_RTLD_NOW
 #pragma GCC poison PETSC_PETSC_HAVE_SCALAPACK
-#pragma GCC poison PETSC_PETSC_HAVE_SCOTCH_PARMETIS_V3_NODEND
 #pragma GCC poison PETSC_PETSC_HAVE_SETJMP_H
 #pragma GCC poison PETSC_PETSC_HAVE_SHMGET
 #pragma GCC poison PETSC_PETSC_HAVE_SLEEP
@@ -136,14 +139,17 @@
 #pragma GCC poison PETSC_PETSC_HAVE_TGAMMA
 #pragma GCC poison PETSC_PETSC_HAVE_TIME
 #pragma GCC poison PETSC_PETSC_HAVE_TIME_H
+#pragma GCC poison PETSC_PETSC_HAVE_TRIANGLE
 #pragma GCC poison PETSC_PETSC_HAVE_UNAME
 #pragma GCC poison PETSC_PETSC_HAVE_UNISTD_H
 #pragma GCC poison PETSC_PETSC_HAVE_USLEEP
 #pragma GCC poison PETSC_PETSC_HAVE_VA_COPY
 #pragma GCC poison PETSC_PETSC_HAVE_VSNPRINTF
+#pragma GCC poison PETSC_PETSC_HAVE_X
 #pragma GCC poison PETSC_PETSC_HAVE_XMMINTRIN_H
 #pragma GCC poison PETSC_PETSC_HAVE_YAML
 #pragma GCC poison PETSC_PETSC_HAVE_ZFP
+#pragma GCC poison PETSC_PETSC_HAVE_ZOLTAN
 #pragma GCC poison PETSC_PETSC_USE_AVX512_KERNELS
 #pragma GCC poison PETSC_PETSC_USE_CTABLE
 #pragma GCC poison PETSC_PETSC_USE_DEBUG
